import ReactDOM from 'react-dom';
import React from 'react';
import PropTypes from 'prop-types';
import CoreApp from 'fusion-core';
import { prepare, PreparePlugin } from 'fusion-react-async';

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

/* eslint-env node */

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

/* eslint-env browser */
var clientRender = (el => {
  const domElement = document.getElementById('root');
  return ReactDOM.hydrate ? ReactDOM.hydrate(el, domElement) : ReactDOM.render(el, domElement);
});

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var Provider = {
  create: name => {
    class Provider extends React.Component {
      getChildContext() {
        return {
          [name]: this.props.provides
        };
      }

      render() {
        return React.Children.only(this.props.children);
      }

    }

    Provider.childContextTypes = Object.assign({}, Provider.childContextTypes || {}, {
      [name]: PropTypes.any.isRequired
    });
    Provider.displayName = name.replace(/^./, c => c.toUpperCase()) + 'Provider';
    return Provider;
  }
};

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var plugin = {
  create: (name, plugin, provider) => {
    let originalMiddleware = plugin.middleware;
    const ProviderComponent = provider || Provider.create(name);

    plugin.middleware = (deps, provides) => {
      if (originalMiddleware) {
        originalMiddleware = originalMiddleware(deps, provides);
      }

      return function (ctx, next) {
        if (ctx.element) {
          ctx.element = React.createElement(ProviderComponent, {
            provides,
            ctx
          }, ctx.element);
        }

        if (originalMiddleware) {
          return originalMiddleware(ctx, next);
        }

        return next();
      };
    };

    return plugin;
  }
};

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var hoc = {
  create: (name, mapProvidesToProps) => {
    if (!mapProvidesToProps) {
      mapProvidesToProps = provides => ({
        [name]: provides
      });
    }

    return Component => {
      class HOC extends React.Component {
        constructor(props, ctx) {
          super(props, ctx);
          this.provides = ctx[name];
        }

        render() {
          const props = Object.assign({}, this.props, mapProvidesToProps(this.provides));
          return React.createElement(Component, props);
        }

      }

      const displayName = Component.displayName || Component.name;
      HOC.displayName = 'With' + name.replace(/^./, c => c.toUpperCase()) + '(' + displayName + ')';
      HOC.contextTypes = {
        [name]: PropTypes.any.isRequired
      };
      return HOC;
    };
  }
};

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

/* eslint-env browser */
class App extends CoreApp {
  constructor(root, render) {
    super(root, el => {
      return prepare(el).then(() => {
        if (render) {
          return render(el);
        }

        return clientRender(el);
      });
    });
    this.middleware(PreparePlugin());
  }

}

export default App;
export { plugin as ProviderPlugin, hoc as ProvidedHOC, Provider };
//# sourceMappingURL=browser.es2015.es.js.map
